/* 
 * Copyright 2007 TKK/Netlab
 * Released under GPLv3. See LICENSE.txt for details. 
 */
package routing;

import core.Connection;
import core.Settings;
import core.SimClock;

/**
 * Passive router that doesn't send anything unless commanded. This is useful
 * for external event -controlled routing or dummy nodes.
 * For implementation specifics, see MessageRouter class.
 */
public class PassiveRouter extends MessageRouter {

	public PassiveRouter(Settings s) {
		super(s);
	}

	/**
	 * Copy-constructor.
	 * @param r Router to copy the settings from.
	 */
	protected PassiveRouter(PassiveRouter r) {
		super(r);
	}

	@Override
	public void update() {
		// nothing to update..
	}

	@Override
	public void changedConnection(Connection con) {
            if (con.isUp())
                System.out.println(SimClock.getTime() + "  " + getHost().getAddress() + "  " + con.getOtherNode(getHost()).getAddress() + " MEET");
	}

	@Override
	public MessageRouter replicate() {
		return new PassiveRouter(this);
	}
}
