#! /usr/bin/perl

package Toolkit;

# ICDF gnuplot plotter

use strict;
use warnings;
use Getopt::Long;

my ($outfile, $labelRe, $term, $range, $params);

my $usage = '
usage:  -out <output file name> [-label <plot title extracting RE>]
        [-params <params for plots>]
        [-term <gnuplot output terminal>]
        [-range <x range of plot>]
        fileNames...
';

GetOptions("label=s" => \$labelRe, "params=s" => \$params,
	   "out=s" => \$outfile, "term=s" => \$term, 
	   "range=s" => \$range);

if (not defined $outfile or not @ARGV) {
    print "Missing required parameter(s)\n";
    print $usage;
    exit();
}

$term = "emf" unless defined $term;
$params = "smooth unique" unless defined $params;
$labelRe = '([^_]*)_' unless defined $labelRe;

my $plotfile = "$outfile.gnuplot";

open(PLOT, ">$plotfile") or die "Can't open plot output file $plotfile : $!";
print PLOT "set logscale x\n";
print PLOT "set ylabel \"1-P(X <= x)\"\n";
if (defined($range)) {
    print PLOT "set xrange [$range]\n";
}
print PLOT "set terminal $term\n";
print PLOT "plot ";

my $round = 0;
while (my $infile = shift(@ARGV)) {
    if ($round > 0) {
	print PLOT ", ";
    }
    $round++;

    open(INFILE, "$infile") or die "Can't open $infile : $!";

    # strip 1-3 char extension (if any)
    if ($infile =~ m/.*\.\w{1,3}/) {
	($infile) = $infile =~ m/(.*)\.\w{1,3}/;
    }

    my $cdffile = "$infile.icdf";
    
    my $totalSum = 0;
    my $hitcount;
    my $time;

    open(OUTFILE,">$cdffile") or die "Can't open output file $outfile : $!";

    while (<INFILE>) {
	($hitcount) = m/\d+.\d+ (\d+)/;
	$totalSum += $hitcount;
    }

    seek(INFILE,0,0);

    my $cumSum = 0;

    while (<INFILE>) {
	($time, $hitcount) = m/(\d+.\d+) (\d+)/;
	if ($hitcount > 0) {
	    $cumSum += $hitcount;
	    print OUTFILE "$time ",1 - ($cumSum / $totalSum),"\n";
	}
    }

    close(OUTFILE);
    close(INFILE);

    print PLOT "'$cdffile'";
    if (defined($labelRe)) { # extract label for legend
	my @labels = $infile =~ m/$labelRe/;
	die "Cant' extract labels using \'$labelRe\' from $infile" unless @labels;
	print PLOT " title \"@labels\"";
    }
    print PLOT " $params";

}

print PLOT "\n";
close(PLOT);

system("gnuplot $plotfile > $outfile") == 0 or die "Error running gnuplot: $?";
